package ejblogin;

import java.security.Principal;
import java.util.Arrays;
import java.util.Set;
import java.util.logging.Logger;

import javax.security.auth.login.LoginException;

import com.sun.appserv.security.AppservPasswordLoginModule;

/**
 * 
 * @author kor
 *
 */
public class EjbLoginModule extends AppservPasswordLoginModule {

	private final Logger LOG = Logger.getLogger(EjbLoginModule.class.getName());
	
	@Override
	protected void authenticateUser() throws LoginException {
	  LOG.info("Authenticate: " + _username);
    
    if (_currentRealm != null && _currentRealm instanceof EjbRealm) {
      
      LOG.info("Found Secured Realm");
      
      EjbRealm cr = (EjbRealm) _currentRealm;
      String[] grpList = cr.authenticate(_username, _password);
      String[] grpListToForward = grpList.clone();
      if (grpList != null) {
        addPrincipal(_username, grpListToForward);
        commitUserAuthentication(grpListToForward);
      } else {
        throw new LoginException("customrealm:Login Failed with user  "
            +_username);
      }
      
    } else {
      throw new LoginException("_currentRealm is not of expected type ["
          + EjbRealm.class.getSimpleName() + ", but ["
          + _currentRealm + "]");
    }
	}
	
	private void addPrincipal(String name, String[] roles) {
	  
	  String[] copyRoles = Arrays.copyOf(roles, roles.length);
		Set<Principal> principals = getSubject().getPrincipals();
		LOG.info("Add secured principal");
		principals.add(new EjbPrincipal(name, copyRoles));
	}
}

